'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _gulpyBoilerUtils = require('gulpy-boiler-utils');

var _eslintFriendlyFormatter = require('eslint-friendly-formatter');

var _eslintFriendlyFormatter2 = _interopRequireDefault(_eslintFriendlyFormatter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Typical `gulp-boiler` task function
 * @param {Object} gulp Gulp instance
 * @param {Object} plugins plugins gathered from `gulp-load-plugins`
 * @param {Object} config configuration utilities
 * @return {Function} to be registered with gulp task
 */
class Eslint extends _gulpyBoilerUtils.TaskHandler {
  constructor(name, plugins, config) {
    super(name, plugins, config);

    const environment = config.environment;
    const utils = config.utils;
    const isLocalDev = environment.isLocalDev;
    const addbase = utils.addbase;
    const getTarget = utils.getTarget;

    const target = getTarget({ name });
    const rootDir = _path2.default.resolve(__dirname, '..', '..', '*');
    let src, base;

    if (target === 'test') {
      base = _path2.default.join(rootDir, 'test/**/*.js');
      src = [addbase('test', '**/*.js')];
    } else if (target === 'build') {
      base = _path2.default.join(rootDir, 'src/**/*.js');
      src = [addbase('tasks', '**/*.js'), addbase('gulpfile.babel.js')];
    }

    /**
     * Lint internal files if developing locally with Lerna
     */
    if (isLocalDev) {
      src.push(base);
    }

    const sources = { src, base, target };

    this.configure({ sources });
    this.loadPlugins({
      config: _path2.default.resolve(__dirname, '..', 'package.json')
    });
  }
  task(gulp, plugins, config) {
    const environment = config.environment;
    const sources = config.sources;
    const src = sources.src;
    const target = sources.target;
    const eslint = plugins.eslint;
    const gulpIf = plugins.gulpIf;
    const isDev = environment.isDev;
    const isLocalDev = environment.isLocalDev;

    const basePath = isLocalDev ? _path2.default.resolve(__dirname, '..', '..', 'eslint-config-gulpy-boiler', 'dist', 'index.js') : require.resolve('eslint-config-gulpy-boiler');
    const getConfig = fp => {
      return fp ? _path2.default.join(_path2.default.dirname(basePath), fp) : basePath;
    };

    return () => {
      let configFile;

      switch (target) {
        case 'test':
          configFile = getConfig('ava.js');
          break;
        case 'build':
          configFile = getConfig('build.js');
          break;
        default:
          configFile = getConfig();
          break;
      }

      const eslintConfig = { configFile };

      this.emit('change', eslintConfig);

      return gulp.src(src).pipe(eslint(eslintConfig)).pipe(eslint.format(_eslintFriendlyFormatter2.default)).pipe(gulpIf(!isDev, eslint.failAfterError()));
    };
  }
}
exports.default = Eslint;
module.exports = exports['default'];